package healthcheck

import (
	"context"
	"fmt"
	"io/ioutil"
	"net/http"
	"strings"
)

// HTTPCheck Defines a generic HTTP check
type HTTPCheck struct {
	*module
}

func (hc *HTTPCheck) check(ctx context.Context, targetIP string, targetPort int) (passed bool, err error) {
	targetURL := fmt.Sprintf("http://%s:%d%s", targetIP, targetPort, hc.Route)
	requestCtx, cancel := context.WithTimeout(ctx, hc.Timeout)
	defer cancel()

	req, err := http.NewRequestWithContext(requestCtx, hc.Method, targetURL, strings.NewReader(hc.Send))
	if err != nil {
		return false, err
	}

	resp, err := http.DefaultClient.Do(req.WithContext(ctx))
	if err != nil {
		return false, err
	}
	defer resp.Body.Close()
	// Check for valid status codes
	passed = false
	for _, code := range hc.StatusCodes {
		if code == resp.StatusCode {
			passed = true
			break
		}
	}
	if !passed {
		return passed, nil
	}
	if hc.Expect == nil {
		return true, nil
	}
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return false, err
	}
	return hc.Expect.Match(body), nil
}
